aws_access_key: {{ session_access_key | default(aws_access_key) }}
aws_secret_key: {{ session_secret_key | default(aws_secret_key) }}
security_token: {{ session_security_token | default(security_token) }}
